package App::GraphQL::Context;

use strict;
use warnings;
use utf8;

use App::GraphQL::DataLoader;

sub new {
    my ($class) = @_;

    return bless +{
        data_loaders => +{},
    }, $class;
}

# DataLoader を作成し，Context で管理できるようにする
sub create_data_loader {
    my ($self, $id, $batch_load_fn) = @_;

    # すでに作成された DataLoader があれば共通のインスタンスを使う
    # なければ，新規で DataLoader を作成しキャッシュする
    my $loader = $self->{data_loaders}->{$id};
    unless (defined $loader) {
        $loader = App::GraphQL::DataLoader->new($batch_load_fn);
        $self->{data_loaders}->{$id} = $loader;
    }

    return $loader;
}

# DataLoader の遅延評価を実行する
sub dispatch_data_loaders {
    my ($self) = @_;

    # TODO: 本当はここでキューがなくなるまで DataLoader の dispatch を継続しないといけない
    for my $data_loader (values $self->{data_loaders}->%*) {
        $data_loader->dispatch();
    }
}

1;
