package App::GraphQL::DataLoader;

use strict;
use warnings;
use utf8;

use Promise::XS;

sub new {
    my ($class, $batch_load_fn) = @_;

    return bless +{
        batch_load_fn => $batch_load_fn,
        batch_map     => +{},
    }, $class;
}

# DataLoader を使った指定のキーのオブジェクトの取得
sub load {
    my ($self, $key) = @_;
    
    # 既に指定のキーに対する DataLoader の結果の Promise があれば，それが使える
    my $deferred = $self->{batch_map}->{$key};

    # 結果は Promise として返し，バッチ取得後にそれぞれのキーに対する結果を遅延して伝える
    unless (defined $deferred) {
        $deferred = Promise::XS::deferred();
        $self->{batch_map}->{$key} = $deferred;
    }

    return $deferred->promise();
}

# 実際に DataLoader に集まったキーを使ってバッチでデータ取得を行う
sub dispatch {
    my ($self) = @_;

    # 集まったキーを適当な単位でバッチ取得を行う
    my $keys = [keys $self->{batch_map}->%*];
    my $batch_result_map = eval { $self->{batch_load_fn}->($keys) } // +{};
    my $error = $@;
    
    for my $key ($keys->@*) {
        my $result = $batch_result_map->{$key};
        my $deferred = $self->{batch_map}->{$key};
        if ($error) {
            $deferred->reject($error);
        } else {
            $deferred->resolve($result);
        }
    }
}

1;
