package App::GraphQL::Resolver::Query;

use strict;
use warnings;
use utf8;

use App::Repository::Book;
use App::Repository::Author;
use App::Repository::Review;

# id による Book 取得を行う DataLoader
sub book_by_id_data_loader {
    my ($class, $context) = @_;
    my $batch_load_fn = sub {
        my ($ids) = @_;
        my $result_map = App::Repository::Book->get_by_ids($ids);

        return $result_map;
    };

    return $context->create_data_loader('book_by_id', $batch_load_fn);
}

# id による Author 取得を行う DataLoader
sub author_by_id_data_loader {
    my ($class, $context) = @_;
    my $batch_load_fn = sub {
        my ($ids) = @_;
        my $result_map = App::Repository::Author->get_by_ids($ids);

        return $result_map;
    };

    return $context->create_data_loader('author_by_id', $batch_load_fn);
}

# id による Review 取得を行う DataLoader
sub review_by_id_data_loader {
    my ($class, $context) = @_;
    my $batch_load_fn = sub {
        my ($ids) = @_;
        my $result_map = App::Repository::Review->get_by_ids($ids);

        return $result_map;
    };

    return $context->create_data_loader('review_by_id', $batch_load_fn);
}

# Query.book の Resolver 実装
sub book {
    my ($class, $root_value, $args, $context, $info) = @_;
    
    return $class->book_by_id_data_loader($context)->load($args->{id});
}

sub node {
    my ($class, $root_value, $args, $context, $info) = @_;
    my $id = $args->{id};
    my ($type, ) = split '::', $id;

    if ($type eq 'Book') {
        return $class->book_by_id_data_loader($context)->load($id);
    } elsif ($type eq 'Author') {
        return $class->author_by_id_data_loader($context)->load($id);
    } elsif ($type eq 'Review') {
        return $class->review_by_id_data_loader($context)->load($id);
    }

    die "TODO";
}

1;
