#!/usr/bin/env perl

use strict;
use warnings;
use utf8;
use lib 'lib';

use App::GraphQL;
use JSON::XS qw(decode_json encode_json);
use Plack::Request;

sub graphql_post {
  my ($req) = @_;
  my $body = eval { decode_json($req->content) } // +{};
  return App::GraphQL->execute_with_cache($body);
}

sub graphql_get {
  my ($req) = @_;
  my $extensions = decode_json($req->query_parameters->{extensions} // '{}');
  my $operation_name = $req->query_parameters->{operationName} // '';
  my $variables = decode_json($req->query_parameters->{variables} // '{}');
  return App::GraphQL->execute_persisted_query($operation_name, $variables, $extensions);
}

my $app = sub {
  my ($env) = @_;
  my $req = Plack::Request->new($env);
  my $method = $env->{REQUEST_METHOD};
  my $result = ($method eq 'POST') ? graphql_post($req) : graphql_get($req);
  my $result_as_json = encode_json($result);

  return [
    200,
    ['Content-Type'=>'application/json'],
    [$result_as_json]
  ];
};

$app;
